/* context.h--general context interface
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef CONTEXT_H
#define CONTEXT_H

#include "../util/str.h"
#include "../util/refcount.h"
#include "vent_vec.h"

typedef struct context_s {
  pthread_mutex_t mutex;
  refcount_t      refcount;
  enum {
    CONTEXT_VALID,
    CONTEXT_INVALID
  }               validity;
  struct client_s *client;
  vent_vec_t     *vec;
  str_t          *modtime;
  const str_t    *name;
  struct context_s *previous;
}               context_t;

#define context_GET_VEC(c) ((c)->vec)
#define context_SET_VEC(c, v) ((c)->vec = (v))
#define context_PREVIOUS(c) ((c)->previous)
#define context_MODTIME(c) ((c)->modtime)
#define context_SET_MODTIME(c, m) ((c)->modtime = *(m), *(m) = NULL, OK)

/* A context is your standard ACAP context.
 * It keeps track of a vent_vec, handles notifications,
 * that sort of thing. */

/* Allocates an unlocked context with no references. */
extern          result_t
context_alloc(struct client_s * client,
	      context_t ** dest);

/* Standard add-ref, remove-ref functions. */
extern result_t context_ref(context_t * context);

extern result_t context_unref(context_t ** context);

/* Locks a context so it may be manipulated.
 * May return ACAP_ERR_NO_SUCH_CONTEXT. */
extern result_t context_lock(context_t * context);

/* Context must be locked.  This unlocks the context, and if
 * invalidate is TRUE, causes all subsequent calls to the context
 * (except for context_unref) to return ACAP_ERR_NO_SUCH_CONTEXT,
 * until the context runs out of references and goes away. */
typedef enum {
  CONTEXT_KEEP,
  CONTEXT_INVALIDATE
}               context_unlock_method_t;

extern result_t
context_unlock(context_t * context,
	       context_unlock_method_t method);

/* This call is used by the client code when a name->context mapping
 * is established, so that if the context is later deemed to be
 * invalid, it can be replaced by a previous version.  This transfers
 * the reference to the previous context from the client to the new
 * context. */
extern result_t
context_set_previous(context_t * context,
		     const str_t * name,
		     context_t * previous);

#endif				/* CONTEXT_H */
