/* dataset_data.h - dataset data interface
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef DATASET_DATA_H
#define DATASET_DATA_H

#include "../util/sllist.h"
#include "entry.h"
#include "trx.h"

struct dataset_ref_s;

typedef struct dataset_data_s dataset_data_t;

extern          result_t dataset_data_set_cachesize(unsigned newsize);

#if defined(DATASET_DATA_C) || defined(DATASET_REF_C)
/* Allocate a new dataset_data object, complete with refcount == one. */
extern result_t
dataset_data_alloc(char *path,
		   struct dataset_ref_s * dataset_ref,
		   dataset_data_t ** dest);

/* This should only be called by dataset_ref, when it's
 * determined that it's really okay to free a dataset_data object;
 * most other things should just use dataset_data_ref() and
 * dataset_data_unref(). */
extern result_t dataset_data_free(dataset_data_t ** data);

#endif				/* defined(DATASET_DATA_C) ||

				 * 
				 * 
				 * 
				 * defined(DATASET_REF_C) */

extern result_t dataset_data_ref(dataset_data_t * data);

extern result_t dataset_data_unref(dataset_data_t ** data);

/* Every time an entry is retrieved via get_entry, done_with_entry
 * should be called when the entry is no longer needed, even if
 * replace_entry is used to replace it. */
extern result_t
dataset_data_get_entry(dataset_data_t * data,
		       trx_t * trx,
		       str_t * name,
		       entry_t ** dest);

extern result_t
dataset_data_done_with_entry(dataset_data_t * data,
			     entry_t ** entry);


extern result_t dataset_data_sync(dataset_data_t * data);

extern result_t
dataset_data_replace_entry(dataset_data_t * data,
			   trx_t * trx,
			   str_t * oldname,
			   entry_t * oldentry,
			   str_t * newname,
			   entry_t * newentry);


/* I'm calling this "stream" instead of "iterator" because
 * it's not quite the same as simply walking the contents
 * of a dataset_data object; what's produced is a sequence
 * of values.  It's also imperative that a stream be disposed
 * of correctly, instead of just magically going away (like
 * the read-only iterator model suggests).
 */

typedef struct dataset_data_stream_s dataset_data_stream_t;

extern result_t
dataset_data_stream_alloc(dataset_data_t * data,
			  trx_t * trx,
			  dataset_data_stream_t ** stream);

extern result_t
dataset_data_stream_next(dataset_data_stream_t * stream,
			 entry_t ** entry);

extern result_t
                dataset_data_stream_free(dataset_data_stream_t ** stream);

#endif				/* DATASET_DATA_H */
