/* entry_ref.h--entry_ref interface
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef ENTRY_REF_H
#define ENTRY_REF_H

/* entry_ref is used to form the datastructure used for ACAP command
 * interlocking.  dataset_ref objects point at entry_ref objects,
 * and entry_ref objects may be used to look up dataset_ref objects,
 * walking the tree of objects. */

#include "dataset.h"
#include "../util/rselock.h"
#include "../util/str.h"
#include "../util/hash.h"
#include "../util/refcount.h"
#include "dataset.h"
#include "dataset_db.h"
#include "trx.h"

struct dataset_ref_s;

struct entry_ref_history_s;

typedef struct entry_ref_s {
  refcount_t      refcount;
  pthread_mutex_t mutex;
  pthread_cond_t  cond;
  struct entry_ref_history_s *writer;
  hash_entry_t   *hash_entry;
  struct entry_ref_history_s *history;
  struct dataset_ref_s *parent;
}               entry_ref_t;

/* Accessor macros... */

#if defined(ENTRY_REF_C) || defined(DATASET_REF_C)
#define entry_ref_REFCOUNT(e) (&((e)->refcount))
#define entry_ref_HASHENT(e) ((e)->hash_entry)
#endif				/* defined(ENTRY_REF_C) ||

				 * defined(DATASET_REF_C) */

/* The mutex and cond is used for modification */
#define entry_ref_MUTEX(e) (&((e)->mutex))
#define entry_ref_COND(e) (&((e)->cond))
#define entry_ref_WRITER(e) ((e)->writer)

typedef struct entry_ref_history_s *entry_ref_history_iter_t;

#define entry_ref_HIST(e) ((e)->history)

#define entry_ref_HIST_ITER_BEGIN(e,i) (*(i) = entry_ref_HIST(e))
#define entry_ref_HIST_ITER_ATEND(i) (*(i) == NULL)
#define entry_ref_HIST_ITER_INCR(i) (*(i) = entry_ref_history_NEXT(*(i)))
#define entry_ref_HIST_ITER_VALUE(i) (*(i))

#define entry_ref_NAME(e) ((e)->hash_entry			\
			   ? (str_t *)hash_GET_KEY((e)->hash_entry)	\
			   : (str_t *)NULL)

#define entry_ref_PARENT(e) ((e)->parent)

/* Create an entry_ref, with a refcount of 1. */
extern          result_t
entry_ref_alloc(struct dataset_ref_s * parent,
		hash_entry_t * hash_entry,
		const str_t * db_path,
		dataset_key_t key,
		entry_ref_t ** dest);

extern result_t entry_ref_free(entry_ref_t ** entry_ref);

/* Add a reference... */
extern result_t entry_ref_ref(entry_ref_t * entry_ref);

/* This removes a reference from an
 * entry_ref_t, setting the entry to NULL */
extern result_t entry_ref_unref(entry_ref_t ** entry);

/* This returns the parent dataset of this entry,
 * entry, with an incremented reference. */
extern result_t
entry_ref_get_parent_dataset(entry_ref_t * entry,
			     struct dataset_ref_s ** dataset);

#endif				/* ENTRY_REF_H */
