/* vent.h--virtual entry interface
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef VENT_H
#define VENT_H

/* A vent ("virtual entry") is an entry as returned by a search,
 * stored in a context, etc; they exist to manipulate a stack of
 * inheriting entry_t elements as if they were a single entry.  So
 * this is really what *does* inheritance. */

/* The construction/destruction of these is mysteriously managed by
 * vent_vec and search; the routines that other modules may use are
 * only for reading.  */

#include "../util/str.h"
#include "entry.h"

typedef struct {
  unsigned        position;
  str_t          *name;
  entry_t        *entries[ACAP_INHERITANCE_DEPTH];
}               vent_t;

#define vent_POSITION(v) ((v)->position)
#define vent_NAME(v) ((v)->name)

typedef struct {
  const entry_t  *const * it;
  const entry_t  *const * end;
}               vent_iterator_t;

#define vent_ITER_BEGIN(v, i) ((i)->it = (const entry_t * const *)v->entries, \
			       (i)->end = (i)->it + ACAP_INHERITANCE_DEPTH)
#define vent_ITER_ATEND(i) ((i)->end <= (i)->it)
#define vent_ITER_INCR(i) (++(i)->it)
#define vent_ITER_VALUE(i) (*(i)->it)

extern          result_t
vent_lookup_attr(const vent_t * vent,
		 const str_t * attr_name,
		 const attr_t ** attr);

extern result_t
vent_check_visible(const vent_t * vent,
		   bool_t * is_visible);

#endif				/* VENT_H */
