/* io.h - io interface
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef IO_H
#define IO_H

/* The idea: when something's interested in IO on some fd, it calls
 * io_add_handler().  When it's ready for data, it calls
 * io_active_fd(); this causes the handler to be called the next time
 * the fd's readable.  When the connection's closed, io_remove_handler
 * should be called; this cleans things up. */

/* Note that the handler will be called in the same thread which does
 * the loop handling; it should go *quickly*. */

/* Initially, at least one handler should be added, and some thread
 * should call io_loop().  When the last handler goes away (or on some
 * error), io_loop() will return.  */

/* Descriptors should be set to send SIGPOLL when they're readable (a
 * signal's necessary anyway, and this is less prone to failure than
 * racing the select). */

/* Everything's threadsafe, although only one thread should ever call
 * io_loop(). */

/* It's an error to call io_active_fd(), io_idle_fd(), or io_remove_handler()
 * on an fd which hasn't been registered via io_add_handler(). */

/* The handler type... */
typedef void    io_handler_t(int fd, void *rock);

/* Adds a fd->handler mapping. */
extern          result_t
io_add_handler(int fd,
	       io_handler_t * handler,
	       void *rock);

/* Removes the handler for a given fd */
extern result_t io_remove_handler(int fd);

/* Loops for IO; terminates when last handler goes away. */
extern result_t io_loop(void);

#endif				/* IO_H */
