/* queue.c - queue implementation
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#define QUEUE_C
#include <config.h>
#include <stdlib.h>
#include <errno.h>
#include "queue.h"
#include "util_err.h"

struct queue_node_s {
  void           *data;
  queue_node_t   *next;
};

result_t
queue_init(q_t * queue)
{
  I(queue);
  queue->front = queue->back = NULL;
  return OK;
}

result_t
queue_destroy(q_t * queue)
{
  queue_node_t   *node;

  I(queue);

  node = queue->front;
  while (node) {
    queue_node_t   *next = node->next;

    free(node);
    node = next;
  }
  return OK;
}

result_t
queue_push(q_t * queue,
	   void *data)
{
  queue_node_t   *node;

  I(queue);

  node = (queue_node_t *) malloc(sizeof(queue_node_t));
  if (!node)
    return errno;

  node->data = data;
  node->next = NULL;

  if (queue->back) {
    I(!queue->back->next);
    queue->back->next = node;
  }
  queue->back = node;

  if (!queue->front)
    queue->front = node;

  return OK;
}

result_t
queue_pop(q_t * queue,
	  void **data)
{
  queue_node_t   *node;

  I(queue);
  I(data);

  if (!queue->front)
    return QUEUE_EMPTY;

  node = queue->front;
  *data = node->data;
  queue->front = node->next;
  if (!queue->front) {
    I(queue->back == node);
    queue->back = NULL;
  }
  free(node);

  return OK;
}
