/* refcache.h -- refcounted-object cache interface
 * Rob Earhart
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Carnegie Mellon
University not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission.

CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
******************************************************************/

#ifndef REFCACHE_H
#define REFCACHE_H

#include "refcount.h"

/* The idea is that a refcache keeps a reference on some number of
 * refcounted objects; this keeps their refcounts active, so that
 * they're not destroyed immediately, possibly preserving some
 * datastructures which would be useful to keep around. */

typedef struct refcache_s refcache_t;

typedef void    refcache_unref_handler_t(void *rock);

/* Standard allocator function; takes the size of the cache
 * and a handler to be used when an object's refcount reaches
 * zero, and returns a refcache. */
extern          result_t
refcache_alloc(unsigned cachesize,
	       refcache_unref_handler_t * handler,
	       refcache_t ** dest);

/* Standard free function; decrements on the refcounts on everything
 * in the cache. */
extern result_t refcache_free(refcache_t ** cache);

/* Empties the cache; decrements the refcounts on everything
 * currently in the cache. */
extern result_t refcache_erase(refcache_t * cache);

/* refcache_used should be called when an object's been recently used,
 * before it's dereferenced; this keeps it in the cache.  The rock
 * is passed to the refcache's refcache_unref_handler_t when the associated
 * refcount drops to zero; the refcount itself is *not* passed to the
 * handler.
 * Note that the refcounted object may already be in the cache (and also
 * note that uniqueness is based on the refcount_t *, not the rock);
 * in this case, nothing will happen, and OK will be returned. */

extern result_t
refcache_used(refcache_t * cache,
	      refcount_t * refcount,
	      void *rock);

/* Dynamically adjusts a cache's size */
extern result_t
refcache_setcachesize(refcache_t * cache,
		      unsigned cachesize);

#endif				/* REFCACHE_H */
